@@ -1,4 +1,4 @@
-# Copyright 2015, 2017 IBM Corp.
+# Copyright IBM Corp. and contributors
 #
 # All Rights Reserved.
 #
@@ -23,10 +23,10 @@ aixlinux, but with the is_mgmt_partition property set to True.
 The PowerVM Nova Compute service runs on the management partition.
 """
 import glob
-from nova import exception
-from nova import utils
 import os
-from os import path
+
+from nova import exception
+from nova.privsep import path as priv_path
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from pypowervm.tasks import partition as pvm_par
@@ -49,15 +49,6 @@ def mgmt_uuid(adapter):
     return _MP_UUID
 
 
-def _tee_as_root(fpath, payload):
-    """Executes 'echo $payload | sudo tee -a $fpath'.
-
-    :param fpath: The file system path to which to tee.
-    :param payload: The string to pipe to the file.
-    """
-    utils.execute('tee', '-a', fpath, process_input=payload, run_as_root=True)
-
-
 def discover_vscsi_disk(mapping, scan_timeout=300):
     """Bring a mapped device into the management partition and find its name.
 
@@ -81,8 +72,6 @@ def discover_vscsi_disk(mapping, scan_timeout=300):
     :raise UniqueDiskDiscoveryException: If more than one disk appears with the
                                          expected UDID.
     """
-    # TODO(IBM): Support for other host platforms.
-
     # Calculate the Linux slot number from the client adapter slot number.
     lslot = 0x30000000 | mapping.client_adapter.lpar_slot_num
     # We'll match the device ID based on the UDID, which is actually the last
@@ -96,8 +85,8 @@ def discover_vscsi_disk(mapping, scan_timeout=300):
     # This glob should yield exactly one result, but use the loop just in case.
     for scanpath in glob.glob(
             '/sys/bus/vio/devices/%x/host*/scsi_host/host*/scan' % lslot):
-        # echo '- - -' | sudo tee -a /path/to/scan
-        _tee_as_root(scanpath, '- - -')
+        # Writing '- - -' to this sysfs file triggers bus rescan
+        priv_path.writefile(scanpath, 'a', '- - -')
 
     # Now see if our device showed up.  If so, we can reliably match it based
     # on its Linux ID, which ends with the disk's UDID.
@@ -123,7 +112,7 @@ def discover_vscsi_disk(mapping, scan_timeout=300):
                                                   count=len(disks))
 
     # The by-id path is a symlink.  Resolve to the /dev/sdX path
-    dpath = path.realpath(disks[0])
+    dpath = os.path.realpath(disks[0])
     LOG.debug("Discovered VSCSI disk with UDID %(udid)s on slot %(slot)x at "
               "path %(devname)s.",
               {'udid': udid, 'slot': lslot, 'devname': dpath})
@@ -147,7 +136,7 @@ def remove_block_dev(devpath, scan_timeout=10):
                                     afterward.
     """
     # Resolve symlinks, if any, to get to the /dev/sdX path
-    devpath = path.realpath(devpath)
+    devpath = os.path.realpath(devpath)
     try:
         os.stat(devpath)
     except OSError:
@@ -161,7 +150,8 @@ def remove_block_dev(devpath, scan_timeout=10):
     LOG.debug("Deleting block device %(devpath)s from the management "
               "partition via special file %(delpath)s.",
               {'devpath': devpath, 'delpath': delpath})
-    _tee_as_root(delpath, '1')
+    # Writing '1' to this sysfs file deletes the block device and rescans.
+    priv_path.writefile(delpath, 'a', '1')
 
     # The bus scan is asynchronous.  Need to poll, waiting for the device to
     # disappear.  Stop when stat raises OSError (dev file not found) - which is
