@@ -1,4 +1,4 @@
-# Copyright 2015, 2017 IBM Corp.
+# Copyright 2015, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -13,6 +13,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import collections
 import copy
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
@@ -22,10 +23,11 @@ from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as p_exc
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm.volume import driver as v_driver
-from nova_powervm.virt.powervm.volume import volume as volume
+from nova_powervm.virt.powervm.volume import volume
 from pypowervm import const as pvm_const
 from pypowervm import exceptions as pvm_exc
 from pypowervm.tasks import hdisk
+from pypowervm.tasks import partition as pvm_partition
 from pypowervm.utils import transaction as tx
 from pypowervm.wrappers import virtual_io_server as pvm_vios
 
@@ -37,6 +39,58 @@ import six
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 DEVNAME_KEY = 'target_devname'
+_ISCSI_INITIATORS = collections.OrderedDict()
+
+
+def get_iscsi_initiators(adapter, vios_ids=None):
+    """Gets the VIOS iSCSI initiators.
+
+     For the first time invocation of this method after process start up,
+     it populates initiators data for VIOSes (if specified, otherwise it
+     gets active VIOSes from the host) and stores in memory for futher
+     lookup.
+
+    :param adapter: The pypowervm adapter
+    :param vios_ids: List of VIOS ids to get the initiators. If not
+                     specified, a list of active VIOSes for the
+                     host is fetched (but only for the first time)
+                     through the pypowervm adapter.
+    :return: A dict of the form
+             {<vios_id>: <list of initiators>}
+    """
+
+    global _ISCSI_INITIATORS
+
+    def discover_initiator(vios_id):
+
+        # Get the VIOS id lock for initiator lookup
+        @lockutils.synchronized('inititator-lookup-' + vios_id)
+        def _discover_initiator():
+            if (vios_id in _ISCSI_INITIATORS and
+                    _ISCSI_INITIATORS[vios_id]):
+                return
+            else:
+                try:
+                    initiator = hdisk.discover_iscsi_initiator(
+                        adapter, vios_id)
+                    _ISCSI_INITIATORS[vios_id] = initiator
+                except (pvm_exc.ISCSIDiscoveryFailed,
+                        pvm_exc.JobRequestFailed) as e:
+                    # TODO(chhagarw): handle differently based on
+                    # error codes
+                    LOG.error(e)
+
+        _discover_initiator()
+
+    if vios_ids is None and not _ISCSI_INITIATORS:
+        vios_list = pvm_partition.get_active_vioses(adapter)
+        vios_ids = [vios.uuid for vios in vios_list]
+
+    for vios_id in vios_ids or []:
+        discover_initiator(vios_id)
+
+    LOG.debug("iSCSI initiator info: %s" % _ISCSI_INITIATORS)
+    return _ISCSI_INITIATORS
 
 
 class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
@@ -84,7 +138,54 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
                          information to the live_migration command, it
                          should be added to this dictionary.
         """
-        raise NotImplementedError()
+
+        # See the connect_volume for why this is a direct call instead of
+        # using the tx_mgr.feed
+        vios_wraps = pvm_vios.VIOS.get(self.adapter,
+                                       xag=[pvm_const.XAG.VIO_STOR])
+
+        volume_key = 'vscsi-' + self.volume_id
+        for vios_w in vios_wraps:
+            if vios_w.uuid not in self.vios_uuids:
+                continue
+            # Discover the volume on all VIOS's to trigger the
+            # device configuration, which in turn will discover the
+            # LUN associated with the volume. This needs to be
+            # attempted on all VIOS's to determine the VIOS's that
+            # will be servicing IO for this volume.
+            udid = self._discover_volume_on_vios(vios_w)[1]
+            if udid:
+                LOG.debug("Discovered volume udid %(udid)s on vios %(name)s",
+                          dict(udid=udid, name=vios_w.name))
+                mig_data[volume_key] = udid
+
+        if volume_key not in mig_data:
+            LOG.debug("Failed to discover the volume")
+            ex_args = dict(volume_id=self.volume_id,
+                           instance_name=self.instance.name)
+            raise p_exc.VolumePreMigrationFailed(**ex_args)
+
+    def post_live_migration_at_destination(self, mig_data):
+        """This method will update the connection info with the volume udid."""
+
+        volume_key = 'vscsi-' + self.volume_id
+        if volume_key in mig_data:
+            self._set_udid(mig_data[volume_key])
+
+    def post_live_migration_at_source(self, migrate_data):
+        """Performs post live migration for the volume on the source host.
+
+        This method can be used to handle any steps that need to taken on
+        the source host after the VM is on the destination.
+
+        :param migrate_data: volume migration data
+        """
+        # Get the udid of the volume to remove the hdisk for.  We can't
+        # use the connection information because LPM 'refreshes' it, which
+        # wipes out our data, so we use the data from the destination host
+        # to avoid having to discover the hdisk to get the udid.
+        udid = migrate_data.get('vscsi-' + self.volume_id)
+        self._cleanup_volume(udid)
 
     def is_volume_on_vios(self, vios_w):
         """Returns whether or not the volume is on a VIOS.
@@ -94,12 +195,51 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
                  otherwise.
         :return: The udid of the device.
         """
+        if vios_w.uuid not in self.vios_uuids:
+            return False, None
         device_name, udid = self._discover_volume_on_vios(vios_w)
         return (device_name and udid) is not None, udid
 
     def _is_multipath(self):
         return self.connection_info["connector"].get("multipath", False)
 
+    def _get_iscsi_conn_props(self, vios_w, auth=False):
+        """Returns the required iSCSI connection properties."""
+        props = dict()
+        try:
+            data = self.connection_info['data']
+            # For multipath target properties should exist
+            if all([key in data for key in ('target_portals',
+                                            'target_iqns',
+                                            'target_luns')]):
+                props['target_portals'] = data['target_portals']
+                props['target_iqns'] = data['target_iqns']
+                props['target_luns'] = data['target_luns']
+
+            if auth and 'discovery_auth_method' in data:
+                for s in ('method', 'username', 'password'):
+                    k = 'discovery_auth_' + s
+                    props[k] = data[k]
+
+            props['target_portal'] = data['target_portal']
+            props['target_iqn'] = data['target_iqn']
+            props['target_lun'] = data['target_lun']
+
+            # if auth_method is set look for username, password
+            if auth and 'auth_method' in data:
+                props['auth_method'] = data['auth_method']
+                props['auth_username'] = data['auth_username']
+                props['auth_password'] = data['auth_password']
+
+            return props
+
+        except (KeyError, ValueError):
+            # Missing information in the connection info
+            LOG.warning('Failed to retrieve iSCSI connection properties '
+                        'for vios %(vios)s, connection_info=%(cinfo)s',
+                        dict(vios=vios_w.uuid, cinfo=self.connection_info))
+        return None
+
     def _discover_vol(self, vios_w, props):
         portal = props.get("target_portals", props.get("target_portal"))
         iqn = props.get("target_iqns", props.get("target_iqn"))
@@ -132,12 +272,17 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
         :returns: LUN or None
         """
         device_name = udid = None
+        conn_props = self._get_iscsi_conn_props(vios_w, auth=True)
+        if conn_props is None:
+            return None, None
+
+        # Check if multipath, we can directly pass the IQN list to
+        # to the low level driver for volume discovery, else iterate
+        # over the IQN and get the list for discovery.
         if self._is_multipath():
-            device_name, udid = self._discover_vol(
-                vios_w, self.connection_info["data"])
+            device_name, udid = self._discover_vol(vios_w, conn_props)
         else:
-            for props in self._iterate_all_targets(
-                    self.connection_info["data"]):
+            for props in self._iterate_all_targets(conn_props):
                 device_name, udid = self._discover_vol(vios_w, props)
         return device_name, udid
 
@@ -162,20 +307,16 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
         device_name, udid = self._discover_volume_on_vios(vios_w)
         if device_name is not None and udid is not None:
             slot, lua = slot_mgr.build_map.get_vscsi_slot(vios_w, device_name)
-            device_name = '/dev/' + device_name
-            iqn = self.connection_info["data"]["target_iqn"]
-            lun = self.connection_info["data"]["target_lun"]
-            target_name = "ISCSI-%s_%s" % (iqn.split(":")[1], str(lun))
+            volume_id = self.connection_info["data"]["volume_id"]
             # Found a hdisk on this Virtual I/O Server.  Add the action to
             # map it to the VM when the stg_ftsk is executed.
             with lockutils.lock(hash(self)):
                 self._add_append_mapping(
                     vios_w.uuid, device_name, lpar_slot_num=slot, lua=lua,
-                    target_name=target_name, udid=udid)
+                    udid=udid, tag=volume_id)
 
-            # Save the devname for the disk in the connection info.  It is
+            # Save the udid  for the disk in the connection info.  It is
             # used for the detach.
-            self._set_devname(device_name)
             self._set_udid(udid)
 
             LOG.debug('Device attached: %s', device_name,
@@ -223,14 +364,15 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
 
             device_name = None
             try:
-                device_name = self._get_devname()
-
-                if not device_name:
-                    # We lost our bdm data.
+                udid = self._get_udid()
+                if udid:
+                    # Get the device name using UniqueDeviceID Identifier.
+                    device_name = vios_w.hdisk_from_uuid(udid)
 
+                if not udid or not device_name:
                     # If we have no device name, at this point
-                    # we should not continue.  Subsequent scrub code on future
-                    # deploys will clean this up.
+                    # we should not continue.  Subsequent scrub code on
+                    # future deploys will clean this up.
                     LOG.warning(
                         "Disconnect Volume: The backing hdisk for volume "
                         "%(volume_id)s on Virtual I/O Server %(vios)s is "
@@ -262,7 +404,9 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
             with lockutils.lock(hash(self)):
                 self._add_remove_mapping(partition_id, vios_w.uuid,
                                          device_name, slot_mgr)
-                conn_data = self.connection_info["data"]
+                conn_data = self._get_iscsi_conn_props(vios_w)
+                if not conn_data:
+                    return False
                 iqn = conn_data.get("target_iqns", conn_data.get("target_iqn"))
                 portal = conn_data.get("target_portals",
                                        conn_data.get("target_portal"))
